(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.crio = {})));
}(this, (function (exports) { 'use strict';

	function createCommonjsModule(fn, module) {
		return module = { exports: {} }, fn(module, module.exports), module.exports;
	}

	var stringify_1 = createCommonjsModule(function (module, exports) {
	exports = module.exports = stringify;
	exports.getSerialize = serializer;

	function stringify(obj, replacer, spaces, cycleReplacer) {
	  return JSON.stringify(obj, serializer(replacer, cycleReplacer), spaces)
	}

	function serializer(replacer, cycleReplacer) {
	  var stack = [], keys = [];

	  if (cycleReplacer == null) cycleReplacer = function(key, value) {
	    if (stack[0] === value) return "[Circular ~]"
	    return "[Circular ~." + keys.slice(0, stack.indexOf(value)).join(".") + "]"
	  };

	  return function(key, value) {
	    if (stack.length > 0) {
	      var thisPos = stack.indexOf(this);
	      ~thisPos ? stack.splice(thisPos + 1) : stack.push(this);
	      ~thisPos ? keys.splice(thisPos, Infinity, key) : keys.push(key);
	      if (~stack.indexOf(value)) value = cycleReplacer.call(this, key, value);
	    }
	    else stack.push(value);

	    return replacer == null ? value : replacer.call(this, key, value)
	  }
	}
	});
	var stringify_2 = stringify_1.getSerialize;

	/**
	 * @constant {Symbol} __ the value to be used as a placeholder
	 */
	var __ = typeof Symbol === 'function' ? Symbol('curriable placeholder') : 0xedd1;

	/**
	 * @function getArgs
	 *
	 * @description
	 * get the complete args with previous placeholders being filled in
	 *
	 * @param {Arguments} originalArgs the arguments from the previous run
	 * @param {Arguments} nextArgs the arguments from the next run
	 * @returns {Array<*>} the complete list of args
	 */
	var getArgs = function getArgs(originalArgs, nextArgs) {
	  var args = new Array(originalArgs.length);

	  var nextArgsIndex = 0;

	  for (var index = 0; index < originalArgs.length; index++) {
	    args[index] = originalArgs[index] === __ && nextArgsIndex < nextArgs.length ? nextArgs[nextArgsIndex++] : originalArgs[index];
	  }

	  if (nextArgsIndex < nextArgs.length) {
	    for (; nextArgsIndex < nextArgs.length; nextArgsIndex++) {
	      args.push(nextArgs[nextArgsIndex]);
	    }
	  }

	  return args;
	};

	/**
	 * @function hasPlaceholder
	 *
	 * @description
	 * determine if any of the arguments are placeholders
	 *
	 * @param {Arguments} args the args passed to the function
	 * @param {number} arity the arity of the function
	 * @returns {boolean} are any of the args placeholders
	 */
	var hasPlaceholder = function hasPlaceholder(args, arity) {
	  for (var index = 0; index < arity; index++) {
	    if (args[index] === __) {
	      return true;
	    }
	  }

	  return false;
	};

	// utils

	/**
	 * @function curry
	 *
	 * @description
	 * get the method passed as a curriable method based on its parameters
	 *
	 * @param {function} fn the method to make curriable
	 * @param {number} [arity=fn.length] the arity of the curried method
	 * @returns {function(...Array<any>): any} the fn passed as a curried function
	 */
	function curry(fn) {
	  var arity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : fn.length;

	  function curried() {
	    var args = arguments;

	    return args.length >= arity && !hasPlaceholder(args, arity) ? fn.apply(this, args) : function () {
	      return curried.apply(this, getArgs(args, arguments));
	    };
	  }

	  curried.arity = arity;
	  curried.fn = fn;

	  return curried;
	}

	curry.__ = __;

	/**
	 * @function uncurry
	 *
	 * @description
	 * return a function that is the non-curried version of the fn passed
	 *
	 * @param {function} curried the curried function to uncurry
	 * @returns {function} the original fn
	 */
	function uncurry(curried) {
	  return curried.fn;
	}

	curry.uncurry = uncurry;

	/**
	 * @function first
	 *
	 * @description
	 * get the first n number of items from the array as a new array (faster than native splice)
	 *
	 * @param {Array<any>} array the array to get the items from
	 * @param {number} length the length to limit the size to
	 * @returns {Array<any>} the array limited in size
	 */
	var first = function first(array, length) {
	  var newArray = new Array(length);

	  for (var index = 0; index < length; index++) {
	    newArray[index] = array[index];
	  }

	  return newArray;
	};

	/**
	 * @function getCircularValue
	 *
	 * @description
	 * create a method that will get a placeholder for the circular value based
	 * on the value saved in the cache for it
	 *
	 * @param {any} key the key of the object to stringify
	 * @param {any} value the value of the object at key
	 * @param {number} refCount the index of the ref
	 * @returns {string} the circular value
	 */
	var getCircularValue = function getCircularValue(key, value, refCount) {
	  return '[ref-' + refCount + ']';
	};

	/**
	 * @function indexOf
	 *
	 * @description
	 * get the index of the value in the array (faster than native indexOf)
	 *
	 * @param {Array<any>} array the array to get the index of the value at
	 * @param {any} value the value to match
	 * @returns {number} the index of the value in array
	 */
	var indexOf = function indexOf(array, value) {
	  for (var index = 0; index < array.length; index++) {
	    if (array[index] === value) {
	      return index;
	    }
	  }

	  return -1;
	};

	/**
	 * @function createReplacer
	 *
	 * @description
	 * create a replacer method that handles circular values
	 *
	 * @param {function} [replacer] a custom replacer to use for non-circular values
	 * @param {function} [circularReplacer] a custom replacer to use for circular methods
	 * @returns {any} the value to stringify
	 */
	var createReplacer = function createReplacer(replacer, circularReplacer) {
	  var getCircularReplacer = circularReplacer || getCircularValue;
	  var hasReplacer = typeof replacer === 'function';

	  var cache = [],
	      locationOfThis = void 0,
	      locationOfValue = void 0;

	  return function (key, value) {
	    if (cache.length) {
	      locationOfThis = indexOf(cache, this);

	      if (~locationOfThis) {
	        cache = first(cache, locationOfThis + 1);
	      } else {
	        cache[cache.length] = this;
	      }

	      locationOfValue = indexOf(cache, value);

	      if (~locationOfValue) {
	        return getCircularReplacer.call(this, key, value, locationOfValue);
	      }
	    } else {
	      cache[0] = value;
	    }

	    return hasReplacer ? replacer.call(this, key, value) : value;
	  };
	};

	// utils

	/**
	 * @function stringify
	 *
	 * @description
	 * strinigifer that handles circular values
	 *
	 * @param {any} value the value to stringify
	 * @param {function} [replacer] a custom replacer function for stringifying standard values
	 * @param {number} [indent] the number of spaces to indent the output by
	 * @param {function} [circularReplacer] a custom replacer function for stringifying circular values
	 * @returns {string} the stringified output
	 */
	function stringify(value, replacer, indent, circularReplacer) {
	  return JSON.stringify(value, createReplacer(replacer, circularReplacer), indent);
	}

	var _SELF_TAGS, _TOSTRING_TAGS, _TYPEDARRAY_TAGS, _UNPARSEABLE_TAGS;

	/**
	 * @constant {string} CIRCULAR_VALUE
	 */
	var CIRCULAR_VALUE = '~';

	/**
	 * @constant {boolean} HAS_BUFFER_FROM_SUPPORT
	 */
	var HAS_BUFFER_FROM_SUPPORT = typeof Buffer !== 'undefined' && typeof Buffer.from === 'function';

	/**
	 * @constant {boolean} HAS_UINT16ARRAY_SUPPORT
	 */
	var HAS_UINT16ARRAY_SUPPORT = typeof Uint16Array === 'function';

	/**
	 * @constant {RegExp} HTML_ELEMENT_REGEXP
	 */
	var HTML_ELEMENT_REGEXP = /\[object (HTML(.*)Element)\]/;

	/**
	 * @constant {RegExp} SVG_ELEMENT_REGEXP
	 */
	var SVG_ELEMENT_REGEXP = /\[object (SVG(.*)Element)\]/;

	/**
	 * @constant {Array<string>} OBJECT_CLASSES
	 */
	var OBJECT_CLASSES = ['Arguments', 'Array', 'ArrayBuffer', 'Boolean', 'DataView', 'Date', 'DocumentFragment', 'Error', 'Event', 'Float32Array', 'Float64Array', 'Function', 'Generator', 'GeneratorFunction', 'HTMLElement', 'Int8Array', 'Int16Array', 'Int32Array', 'Map', 'Null', 'Number', 'Object', 'Promise', 'RegExp', 'Set', 'String', 'Symbol', 'Uint8Array', 'Uint8ClampedArray', 'Uint16Array', 'Uint32Array', 'Undefined', 'WeakMap', 'WeakSet', 'Window'];

	/**
	 * @constant {Object} OBJECT_CLASS_MAP
	 */
	var OBJECT_CLASS_MAP = OBJECT_CLASSES.reduce(function (objectClasses, type) {
	  objectClasses['[object ' + type + ']'] = type;

	  return objectClasses;
	}, {});

	/**
	 * @constant {Object} OBJECT_CLASS_TYPE_MAP
	 */
	var OBJECT_CLASS_TYPE_MAP = Object.keys(OBJECT_CLASS_MAP).reduce(function (objectClassTypes, objectClass) {
	  objectClassTypes[OBJECT_CLASS_MAP[objectClass].toUpperCase()] = objectClass;

	  return objectClassTypes;
	}, {});

	var ITERABLE_TAGS = {
	  '[object Map]': true,
	  '[object Set]': true
	};

	var PRIMITIVE_TAGS = {
	  boolean: true,
	  function: true,
	  number: true,
	  string: true,
	  undefined: true
	};

	var SELF_TAGS = (_SELF_TAGS = {}, _SELF_TAGS[OBJECT_CLASS_TYPE_MAP.ARGUMENTS] = true, _SELF_TAGS[OBJECT_CLASS_TYPE_MAP.ARRAY] = true, _SELF_TAGS);

	var TOSTRING_TAGS = (_TOSTRING_TAGS = {}, _TOSTRING_TAGS[OBJECT_CLASS_TYPE_MAP.REGEXP] = true, _TOSTRING_TAGS[OBJECT_CLASS_TYPE_MAP.SYMBOL] = true, _TOSTRING_TAGS);

	var TYPEDARRAY_TAGS = (_TYPEDARRAY_TAGS = {}, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.FLOAT32ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.FLOAT64ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.INT8ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.INT16ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.INT32ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.UINT8ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.UINT8CLAMPEDARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.UINT16ARRAY] = true, _TYPEDARRAY_TAGS[OBJECT_CLASS_TYPE_MAP.UINT32ARRAY] = true, _TYPEDARRAY_TAGS);

	var UNPARSEABLE_TAGS = (_UNPARSEABLE_TAGS = {}, _UNPARSEABLE_TAGS[OBJECT_CLASS_TYPE_MAP.GENERATOR] = true, _UNPARSEABLE_TAGS[OBJECT_CLASS_TYPE_MAP.PROMISE] = true, _UNPARSEABLE_TAGS[OBJECT_CLASS_TYPE_MAP.WEAKMAP] = true, _UNPARSEABLE_TAGS[OBJECT_CLASS_TYPE_MAP.WEAKSET] = true, _UNPARSEABLE_TAGS);

	// external dependencies

	var charCodeAt = String.prototype.charCodeAt;
	var keys = Object.keys;

	/**
	 * @function getFunctionName
	 *
	 * @description
	 * get the name of the function based on a series of fallback attempts
	 *
	 * @param {function} fn the function to test
	 * @returns {string} the function name
	 */
	var getFunctionName = function getFunctionName(fn) {
	  return fn.name || (fn.toString().match(/^\s*function\s*([^\(]*)/i) || [])[1] || 'anonymous';
	};

	/**
	 * @function getCircularValue
	 *
	 * @description
	 * get the value used when circular references are found
	 *
	 * @returns {string} the value for stringification
	 */
	var getCircularValue$1 = function getCircularValue() {
	  return CIRCULAR_VALUE;
	};

	/**
	 * @function getIntegerHashValue
	 *
	 * @description
	 * based on string passed, get the integer hash value
	 * through bitwise operation (based on spinoff of dbj2
	 * with enhancements for reduced collisions)
	 *
	 * @param {string} string the string to get the hash value for
	 * @returns {number} the hash value
	 */
	var getIntegerHashValue = function getIntegerHashValue(string) {
	  var index = string.length,
	      hashA = 5381,
	      hashB = 52711,
	      charCode = void 0;

	  while (index--) {
	    charCode = charCodeAt.call(string, index);

	    hashA = hashA * 33 ^ charCode;
	    hashB = hashB * 33 ^ charCode;
	  }

	  return (hashA >>> 0) * 4096 + (hashB >>> 0);
	};

	/**
	 * @function getSortedEvent
	 *
	 * @description
	 * get the event object sorted by its properties
	 *
	 * @param {boolean} bubbles does the event bubble up through the DOM
	 * @param {function} alias to stopPropagation
	 * @param {boolean} cancelable is the event cancelable
	 * @param {boolean} composed can the event bubble across the boundary to shadow DOM
	 * @param {HTMLElement} [currentTarget] registered target for the event
	 * @param {boolean} defaultPrevented has preventDefault been called on the event
	 * @param {string} eventPhase the phase of the event flow being processed
	 * @param {boolean} isTrusted was the event initiated by the browser
	 * @param {HTMLElement} [target] the target with which the event was dispatched
	 * @param {number} timeStamp the time at which the event was created
	 * @param {string} type the name of the event
	 * @returns {Object} the event object with all properties sorted
	 */
	var getSortedEvent = function getSortedEvent(_ref) {
	  var bubbles = _ref.bubbles,
	      cancelBubble = _ref.cancelBubble,
	      cancelable = _ref.cancelable,
	      composed = _ref.composed,
	      currentTarget = _ref.currentTarget,
	      defaultPrevented = _ref.defaultPrevented,
	      eventPhase = _ref.eventPhase,
	      isTrusted = _ref.isTrusted,
	      returnValue = _ref.returnValue,
	      target = _ref.target,
	      type = _ref.type;
	  return {
	    bubbles: bubbles,
	    cancelBubble: cancelBubble,
	    cancelable: cancelable,
	    composed: composed,
	    currentTarget: currentTarget,
	    defaultPrevented: defaultPrevented,
	    eventPhase: eventPhase,
	    isTrusted: isTrusted,
	    returnValue: returnValue,
	    target: target,
	    type: type
	  };
	};

	/**
	 * @function shouldSort
	 *
	 * @description
	 * get the sort result based on the two values to compare
	 *
	 * @param {string} valueA the first value to compare
	 * @param {string} valueB the second value to compare
	 * @returns {boolean} should the value be sorted
	 */
	var shouldSort = function shouldSort(valueA, valueB) {
	  return valueA > valueB;
	};

	/**
	 * @function shouldSortPair
	 *
	 * @description
	 * get the sort result based on the two pairs to compare
	 *
	 * @param {Object} pairA the first pair to compare
	 * @param {Object} pairB the second pair to compare
	 * @returns {boolean} should the value be sorted
	 */
	var shouldSortPair = function shouldSortPair(pairA, pairB) {
	  return shouldSort(pairA[0], pairB[0]);
	};

	/**
	 * @function getPrefixedValue
	 *
	 * @description
	 * get the value prefixed by the tag
	 *
	 * @param {string} tag the object tag
	 * @param {any} value the value to stringify
	 * @returns {string} the prefixed stringified value
	 */
	var getPrefixedValue = function getPrefixedValue(tag, value) {
	  return tag + '|' + value;
	};

	/**
	 * @function sort
	 *
	 * @description
	 * sort the array based on the fn passed
	 *
	 * @param {Array<any>} array the array to sort
	 * @param {function} fn the sorting function
	 * @returns {Array<any>} the sorted array
	 */
	var sort = function sort(array, fn) {
	  var subIndex = void 0,
	      value = void 0;

	  for (var index = 0; index < array.length; index++) {
	    value = array[index];

	    for (subIndex = index - 1; ~subIndex && fn(array[subIndex], value); subIndex--) {
	      array[subIndex + 1] = array[subIndex];
	    }

	    array[subIndex + 1] = value;
	  }

	  return array;
	};

	/**
	 * @function getIterablePairs
	 *
	 * @description
	 * get the pairs in the iterable for stringification
	 *
	 * @param {Map|Set} iterable the iterable to get the pairs for
	 * @returns {Array<{key: string, value: any}>} the pairs
	 */
	var getSortedIterablePairs = function getSortedIterablePairs(iterable) {
	  var isMap = typeof iterable.get === 'function';
	  var pairs = [];

	  iterable.forEach(function (value, key) {
	    // eslint-disable-next-line no-use-before-define
	    pairs.push(isMap ? [stringify$1(key), stringify$1(value)] : [stringify$1(value)]);
	  });

	  sort(pairs, shouldSortPair);

	  var finalPairs = new Array(iterable.size);

	  var pair = void 0;

	  for (var index = 0; index < iterable.size; index++) {
	    pair = pairs[index];

	    finalPairs[index] = isMap ? '[' + pair[0] + ',' + pair[1] + ']' : pair[0];
	  }

	  return getPrefixedValue(getFunctionName(iterable.constructor), '[' + finalPairs.join(',') + ']');
	};

	/**
	 * @function getSortedObject
	 *
	 * @description
	 * get the object with the keys sorted
	 *
	 * @param {Object} object the object to sort
	 * @returns {Object} the sorted object
	 */
	var getSortedObject = function getSortedObject(object) {
	  var objectKeys = sort(keys(object), shouldSort);
	  var newObject = {};

	  var key = void 0;

	  for (var index = 0; index < objectKeys.length; index++) {
	    key = objectKeys[index];

	    newObject[key] = object[key];
	  }

	  return newObject;
	};

	/**
	 * @function getStringifiedArrayBufferFallback
	 *
	 * @description
	 * get the string value of the buffer passed based on a Buffer
	 *
	 * @param {ArrayBuffer} buffer the array buffer to convert
	 * @returns {string} the stringified buffer
	 */
	var getStringifiedArrayBufferFallback = function getStringifiedArrayBufferFallback(buffer) {
	  return String.fromCharCode.apply(null, new Uint16Array(buffer));
	};

	/**
	 * @function getStringifiedArrayBufferModern
	 *
	 * @description
	 * get the string value of the buffer passed based on a Uint16Array
	 *
	 * @param {ArrayBuffer} buffer the array buffer to convert
	 * @returns {string} the stringified buffer
	 */
	var getStringifiedArrayBufferModern = function getStringifiedArrayBufferModern(buffer) {
	  return Buffer.from(buffer).toString('utf8');
	};

	/**
	 * @function getStringifiedArrayBufferNoSupport
	 *
	 * @description
	 * return a placeholder when no arraybuffer support exists
	 *
	 * @returns {string} the placeholder
	 */
	var getStringifiedArrayBufferNoSupport = function getStringifiedArrayBufferNoSupport() {
	  return '';
	};

	/**
	 * @function getStringifiedArrayBuffer
	 *
	 * @description
	 * get the string value of the buffer passed
	 *
	 * @param {ArrayBuffer} buffer the array buffer to convert
	 * @returns {string} the stringified buffer
	 */
	var getStringifiedArrayBuffer = function () {
	  return HAS_BUFFER_FROM_SUPPORT ? getStringifiedArrayBufferModern : HAS_UINT16ARRAY_SUPPORT ? getStringifiedArrayBufferFallback : getStringifiedArrayBufferNoSupport;
	}();

	/**
	 * @function getStringifiedDocumentFragment
	 *
	 * @description
	 * build a string based on all the fragment's children
	 *
	 * @param {DocumentFragment} fragment the fragment to stringify
	 * @returns {string} the stringified fragment
	 */
	var getStringifiedDocumentFragment = function getStringifiedDocumentFragment(fragment) {
	  var children = fragment.children;

	  var innerHTML = '';

	  for (var index = 0; index < children.length; index++) {
	    // eslint-disable-next-line no-use-before-define
	    innerHTML += children[index].outerHTML;
	  }

	  return innerHTML;
	};

	/**
	 * @function indexOf
	 *
	 * @description
	 * get the index of the value in the array (faster than native indexOf)
	 *
	 * @param {Array<any>} array the array to get the index of the value at
	 * @param {any} value the value to match
	 * @returns {number} the index of the value in array
	 */
	var indexOf$1 = function indexOf(array, value) {
	  for (var index = 0; index < array.length; index++) {
	    if (array[index] === value) {
	      return index;
	    }
	  }

	  return -1;
	};

	/**
	 * @function getNormalizedValue
	 *
	 * @description
	 * get the value normalized for stringification
	 *
	 * @param {any} value the value to normalize
	 * @param {WeakMap|Object} sortedCache the cache of sorted objects
	 * @returns {any} the normalized value
	 */
	var getNormalizedValue = function getNormalizedValue(value, sortedCache) {
	  var type = typeof value;

	  if (type === 'string') {
	    return value;
	  }

	  if (PRIMITIVE_TAGS[type]) {
	    return getPrefixedValue(type, value);
	  }

	  if (value === null) {
	    return getPrefixedValue('null', value);
	  }

	  var tag = toString.call(value);

	  if (SELF_TAGS[tag]) {
	    return value;
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.OBJECT) {
	    if (~indexOf$1(sortedCache, value)) {
	      return CIRCULAR_VALUE;
	    }

	    sortedCache.push(value);

	    return getSortedObject(value, sortedCache);
	  }

	  if (TOSTRING_TAGS[tag]) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], value.toString());
	  }

	  if (ITERABLE_TAGS[tag]) {
	    if (~indexOf$1(sortedCache, value)) {
	      return CIRCULAR_VALUE;
	    }

	    sortedCache.push(value);

	    return getSortedIterablePairs(value);
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.DATE) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], value.getTime());
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.ERROR) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], value.stack);
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.EVENT) {
	    return getSortedEvent(value);
	  }

	  if (UNPARSEABLE_TAGS[tag]) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], 'NOT_ENUMERABLE');
	  }

	  if (HTML_ELEMENT_REGEXP.test(tag) || SVG_ELEMENT_REGEXP.test(tag)) {
	    return getPrefixedValue(tag.slice(8, -1), value.outerHTML);
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.DOCUMENTFRAGMENT) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], getStringifiedDocumentFragment(value));
	  }

	  if (TYPEDARRAY_TAGS[tag]) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], value.join(','));
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.ARRAYBUFFER) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], getStringifiedArrayBuffer(value));
	  }

	  if (tag === OBJECT_CLASS_TYPE_MAP.DATAVIEW) {
	    return getPrefixedValue(OBJECT_CLASS_MAP[tag], getStringifiedArrayBuffer(value.buffer));
	  }

	  return value;
	};

	/**
	 * @function replacer
	 *
	 * @description
	 * create the replacer function used for stringification
	 *
	 * @param {WeakSet|Object} sortedCache the cache to use for sorting objects
	 * @returns {function(key: string, value: any)} function getting the normalized value
	 */
	var createReplacer$1 = function createReplacer(sortedCache) {
	  return function (key, value) {
	    return getNormalizedValue(value, sortedCache);
	  };
	};

	/**
	 * @function stringify
	 *
	 * @description
	 * stringify the value based on the options passed
	 *
	 * @param {any} value the value to stringify
	 * @returns {string} the stringified value
	 */
	function stringify$1(value) {
	  return typeof value === 'object' && value && !(value instanceof RegExp || value instanceof Date) ? stringify(value, createReplacer$1([]), null, getCircularValue$1) : getNormalizedValue(value);
	}

	// external dependencies

	/**
	 * @function hash
	 *
	 * @description
	 * hash the value passed to a unique, consistent hash value
	 *
	 * @param {any} value the value to hash
	 * @returns {number} the object hash
	 */
	var hash = function hash(value) {
	  return getIntegerHashValue(stringify$1(value));
	};

	/**
	 * @function hash.is
	 *
	 * @description
	 * create a comparator for the first object passed to determine if the second is equal
	 *
	 * @param {any} object the object to test against
	 * @returns {function(any): boolean} the method to test against the object
	 */
	hash.is = curry(function (object, otherObject) {
	  return hash(object) === hash(otherObject);
	});

	/**
	 * @function hash.is.all
	 *
	 * @description
	 * determine if all of the objects passed are equal in value to the first
	 *
	 * @param {...Array<any>} objects the objects to test for equality
	 * @returns {boolean} are the objects equal
	 */
	hash.is.all = curry(function () {
	  for (var _len = arguments.length, objects = Array(_len), _key = 0; _key < _len; _key++) {
	    objects[_key] = arguments[_key];
	  }

	  var isEqual = hash.is(objects.shift());

	  for (var index = 0; index < objects.length; index++) {
	    if (!isEqual(objects[index])) {
	      return false;
	    }
	  }

	  return true;
	}, 2);

	/**
	 * @function hash.is.any
	 *
	 * @description
	 * determine if any of the objects passed are equal in value to the first
	 *
	 * @param {...Array<any>} objects the objects to test for equality
	 * @returns {boolean} are the objects equal
	 */
	hash.is.any = curry(function () {
	  for (var _len2 = arguments.length, objects = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
	    objects[_key2] = arguments[_key2];
	  }

	  var isEqual = hash.is(objects.shift());

	  for (var index = 0; index < objects.length; index++) {
	    if (isEqual(objects[index])) {
	      return true;
	    }
	  }

	  return false;
	}, 2);

	/**
	 * @function hash.is.not
	 *
	 * @description
	 * create a comparator for the first object passed to determine if the second is not equal
	 *
	 * @param {any} object the object to test against
	 * @returns {function(any): boolean} the method to test against the object
	 */
	hash.is.not = curry(function (object, otherObject) {
	  return hash(object) !== hash(otherObject);
	});

	/**
	 * @constant {Object} CACHE
	 *
	 * @property {function} clear clear the cache results
	 * @property {Object} results the map of path => array results
	 * @property {number} size the size of the cache
	 */
	var CACHE = {
	  clear: function clear() {
	    CACHE.results = {};
	    CACHE.size = 0;
	  },

	  results: {},
	  size: 0
	};

	/**
	 * @constant {RegExp} DOTTY_WITH_BRACKETS_SYNTAX
	 */
	var DOTTY_WITH_BRACKETS_SYNTAX = /"[^"]+"|`[^`]+`|'[^']+'|[^.[\]]+/g;

	/**
	 * @constant {number} MAX_CACHE_SIZE
	 */
	var MAX_CACHE_SIZE = 500;

	/**
	 * @constant {RegExp} NUMBER
	 */
	var NUMBER = /^\d+$/i;

	/**
	 * @constant {RegExp} QUOTED_KEY
	 */
	var QUOTED_KEY = /^"[^"]+"|`[^`]+`|'[^']+'$/;

	// constants

	/**
	 * @function isNumericKey
	 *
	 * @description
	 * is the key passed a numeric string
	 *
	 * @param {string} key the key to test
	 * @returns {boolean} is the key passed a numeric string
	 */
	var isNumericKey = function isNumericKey(key) {
	  return !!key.length && NUMBER.test(key);
	};

	/**
	 * @function isQuotedKey
	 *
	 * @description
	 * is the key passed a quoted key
	 *
	 * @param {string} key the key to test
	 * @returns {boolean} is the key a quoted key
	 */
	var isQuotedKey = function isQuotedKey(key) {
	  return QUOTED_KEY.test(key);
	};

	/**
	 * @function getNormalizedParseKey
	 *
	 * @description
	 * get the key as a number if parseable, or as a quoted string if applicable
	 *
	 * @param {string} key the key to try to parse
	 * @returns {number|string} the parsed key
	 */
	var getNormalizedParseKey = function getNormalizedParseKey(key) {
	  var cleanKey = isQuotedKey(key) ? key.substring(1, key.length - 1) : key;

	  return isNumericKey(cleanKey) ? +cleanKey : cleanKey;
	};

	/**
	 * @function parsePath
	 *
	 * @description
	 * parse the path, memoizing the results
	 *
	 * @param {string} path the path to parse
	 * @returns {Array<number|string>} the parsed path
	 */
	var parseStringPath = function parseStringPath(path) {
	  if (CACHE.results[path]) {
	    return CACHE.results[path];
	  }

	  if (CACHE.size > MAX_CACHE_SIZE) {
	    CACHE.clear();
	  }

	  CACHE.results[path] = path ? path.match(DOTTY_WITH_BRACKETS_SYNTAX).map(getNormalizedParseKey) : [path];
	  CACHE.size++;

	  return CACHE.results[path];
	};

	// constants

	/**
	 * @function parse
	 *
	 * @description
	 * the path parsed into a valid array of keys / indices
	 *
	 * @param {Array<number|string>|number|string} path the path to parse
	 * @returns {Array<number|string>} the parsed path
	 */
	var parse = function parse(path) {
	  if (typeof path === 'string') {
	    return parseStringPath(path);
	  }

	  if (Array.isArray(path)) {
	    return path.map(getNormalizedParseKey);
	  }

	  return [typeof path === 'number' ? path : '' + path];
	};

	/**
	 * @constant {Symbol} __ the value to be used as a placeholder
	 */
	var __$1 = typeof Symbol === 'function' ? Symbol('curriable placeholder') : 0xedd1;

	/**
	 * @function getArgs
	 *
	 * @description
	 * get the complete args with previous placeholders being filled in
	 *
	 * @param {Arguments} originalArgs the arguments from the previous run
	 * @param {Arguments} nextArgs the arguments from the next run
	 * @returns {Array<*>} the complete list of args
	 */
	var getArgs$1 = function getArgs(originalArgs, nextArgs) {
	  var argsToPass = new Array(originalArgs.length),
	      nextArgsIndex = 0;

	  for (var index = 0; index < originalArgs.length; index++) {
	    argsToPass[index] = originalArgs[index] === __$1 && nextArgsIndex < nextArgs.length ? nextArgs[nextArgsIndex++] : originalArgs[index];
	  }

	  if (nextArgsIndex < nextArgs.length) {
	    for (; nextArgsIndex < nextArgs.length; nextArgsIndex++) {
	      argsToPass.push(nextArgs[nextArgsIndex]);
	    }
	  }

	  return argsToPass;
	};

	/**
	 * @function hasPlaceholder
	 *
	 * @description
	 * determine if any of the arguments are placeholders
	 *
	 * @param {Arguments} args the args passed to the function
	 * @param {number} arity the arity of the function
	 * @returns {boolean} are any of the args placeholders
	 */
	var hasPlaceholder$1 = function hasPlaceholder(args, arity) {
	  for (var index = 0; index < arity; index++) {
	    if (args[index] === __$1) {
	      return true;
	    }
	  }

	  return false;
	};

	// utils

	/**
	 * @function curry
	 *
	 * @description
	 * get the method passed as a curriable method based on its parameters
	 *
	 * @param {function} fn the method to make curriable
	 * @param {number} [arity=fn.length] the arity of the curried method
	 * @returns {function(...Array<any>): any} the fn passed as a curriable method
	 */
	function curry$1(fn) {
	  var arity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : fn.length;

	  return function curried() {
	    var originalArgs = arguments;

	    return originalArgs.length >= arity && !hasPlaceholder$1(originalArgs, arity) ? fn.apply(this, originalArgs) : function () {
	      return curried.apply(this, getArgs$1(originalArgs, arguments));
	    };
	  };
	}

	curry$1.__ = __$1;

	// external dependencies

	var O = Object;
	var assign = O.assign,
	    create$1 = O.create,
	    getPrototypeOf = O.getPrototypeOf,
	    keys$1 = O.keys;

	/**
	 * @constant {Symbol} REACT_ELEMENT
	 */
	// eslint-disable-next-line no-magic-numbers

	var REACT_ELEMENT = typeof Symbol === 'function' && Symbol.for ? Symbol.for('react.element') : 0xeac7;

	/**
	 * @constant {RegExp} FUNCTION_NAME
	 */
	var FUNCTION_NAME = /^\s*function\s*([^\(]*)/i;

	/**
	 * @function isArray
	 */
	var isArray = Array.isArray;
	var isCloneable = function isCloneable(object) {
	  return !!object && typeof object === 'object' && !(object instanceof Date || object instanceof RegExp) && object.$$typeof !== REACT_ELEMENT;
	};

	/**
	 * @function isGlobalConstructor
	 *
	 * @description
	 * is the function passed a global constructor function
	 *
	 * @param {function} fn the function to test
	 * @returns {boolean} is the function a global constructor
	 */
	var isGlobalConstructor = function isGlobalConstructor(fn) {
	  return typeof fn === 'function' && global[fn.name || Function.prototype.toString.call(fn).split(FUNCTION_NAME)[1]] === fn;
	};

	/**
	 * @function callIfFunction
	 *
	 * @description
	 * call the object passed if it is a function and return its return, else return undefined
	 *
	 * @param {*} object the object to conditionally call if a function
	 * @param {*} context the context to apply to the call
	 * @param {Array<*>} parameters the parametesr to apply the function with
	 * @returns {*} the restulf of the call or undefined
	 */
	var callIfFunction = function callIfFunction(object, context, parameters) {
	  return typeof object === 'function' ? object.apply(context, parameters) : void 0;
	};

	/**
	 * @function getShallowClone
	 *
	 * @description
	 * get a shallow clone of the value passed based on the type requested (maintaining prototype if possible)
	 *
	 * @param {Array<*>|Object} object the object to clone
	 * @param {number|string} key the key to base the object type fromisReactElement(object) ||
	 * @returns {Array<*>|Object} a shallow clone of the value
	 */
	var getShallowClone = function getShallowClone(object) {
	  if (object.constructor === O) {
	    return assign({}, object);
	  }

	  if (isArray(object)) {
	    var newObject = new object.constructor();

	    for (var index = 0; index < object.length; index++) {
	      newObject[index] = object[index];
	    }

	    return newObject;
	  }

	  return isGlobalConstructor(object.constructor) ? {} : keys$1(object).reduce(function (clone, key) {
	    clone[key] = object[key];

	    return clone;
	  }, create$1(getPrototypeOf(object)));
	};

	/**
	 * @function getNewEmptyChild
	 *
	 * @description
	 * get a new empty child for the type of key provided
	 *
	 * @param {number|string} key the key to test
	 * @returns {Array|Object} the empty child
	 */
	var getNewEmptyChild = function getNewEmptyChild(key) {
	  return typeof key === 'number' ? [] : {};
	};

	/**
	 * @function getNewEmptyObject
	 *
	 * @description
	 * get a new empty object for the type of key provided
	 *
	 * @param {Array|Object} object the object to get an empty value of
	 * @returns {Array|Object} the empty object
	 */
	var getNewEmptyObject = function getNewEmptyObject(object) {
	  return isArray(object) ? [] : {};
	};

	/**
	 * @function cloneIfPossible
	 *
	 * @description
	 * clone the object passed if it is mergeable, else return itself
	 *
	 * @param {*} object he object to clone
	 * @returns {*} the cloned object
	 */
	var cloneIfPossible = function cloneIfPossible(object) {
	  return isCloneable(object) ? getShallowClone(object) : object;
	};

	/**
	 * @function getNewChildClone
	 *
	 * @description
	 * get the shallow clone of the child when it is the correct type
	 *
	 * @param {Array<*>|Object} object the object to clone
	 * @param {number|string} nextKey the key that the next object will be based from
	 * @returns {Array<*>|Object} the clone of the key at object
	 */
	var getNewChildClone = function getNewChildClone(object, nextKey) {
	  return isCloneable(object) ? getShallowClone(object) : getNewEmptyChild(nextKey);
	};

	/**
	 * @function getCoalescedValue
	 *
	 * @description
	 * get the value if it is not undefined, else get the fallback
	 *
	 * @param {any} value the main value to return
	 * @param {any} fallbackValue the value to return if main is undefined
	 * @returns {any} the coalesced value
	 */
	var getCoalescedValue = function getCoalescedValue(value, fallbackValue) {
	  return value === void 0 ? fallbackValue : value;
	};

	/**
	 * @function onMatchAtPath
	 *
	 * @description
	 * when there is a match for the path requested, call onMatch, else return the noMatchValue
	 *
	 * @param {Array<number|string>} path the path to find a match at
	 * @param {Array<*>|Object} object the object to find the path in
	 * @param {function} onMatch when a match is found, call this method
	 * @param {boolean} shouldClone should the object be cloned
	 * @param {*} noMatchValue when no match is found, return this value
	 * @param {number} [index=0] the index of the key to process
	 * @returns {*} either the return from onMatch or the noMatchValue
	 */
	var onMatchAtPath = function onMatchAtPath(path, object, onMatch, shouldClone, noMatchValue) {
	  var index = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 0;

	  var key = path[index];
	  var nextIndex = index + 1;

	  if (nextIndex === path.length) {
	    var result = object || shouldClone ? onMatch(object, key) : noMatchValue;

	    return shouldClone ? object : result;
	  }

	  if (shouldClone) {
	    object[key] = onMatchAtPath(path, getNewChildClone(object[key], path[nextIndex]), onMatch, shouldClone, noMatchValue, nextIndex);

	    return object;
	  }

	  return object && object[key] ? onMatchAtPath(path, object[key], onMatch, shouldClone, noMatchValue, nextIndex) : noMatchValue;
	};

	/**
	 * @function getDeeplyMergedObject
	 *
	 * @description
	 * get the objects merged into a new object
	 *
	 * @param {Array<*>|Object} object1 the object to merge into
	 * @param {Array<*>|Object} object2 the object to merge
	 * @returns {Array<*>|Object} the merged object
	 */
	var getDeeplyMergedObject = function getDeeplyMergedObject(object1, object2) {
	  var isObject1Array = isArray(object1);

	  if (isObject1Array !== isArray(object2) || !isCloneable(object1)) {
	    return cloneIfPossible(object2);
	  }

	  if (isObject1Array) {
	    return object1.concat(object2.map(cloneIfPossible));
	  }

	  var target = keys$1(object1).reduce(function (clone, key) {
	    clone[key] = cloneIfPossible(object1[key]);

	    return clone;
	  }, object1.constructor === O ? {} : create$1(getPrototypeOf(object1)));

	  return keys$1(object2).reduce(function (clone, key) {
	    clone[key] = isCloneable(object2[key]) ? getDeeplyMergedObject(object1[key], object2[key]) : object2[key];

	    return clone;
	  }, target);
	};

	/**
	 * @function getParsedPath
	 *
	 * @description
	 * get the path array, either as-is if already an array, or parsed by pathington
	 *
	 * @param {Array<number|string>|number|string} path the path to parse
	 * @returns {Array<number|string>} the parsed path
	 */
	var getParsedPath = function getParsedPath(path) {
	  return isArray(path) ? path : parse(path);
	};

	/**
	 * @function callNestedProperty
	 *
	 * @description
	 * parse the path passed and call the nested method at that path
	 *
	 * @param {Array<number|string>|number|string} path the path to retrieve values from the object
	 * @param {*} context the context that the method is called with
	 * @param {Array<*>} parameters the parameters to call the method with
	 * @param {*} object the object to get values from
	 * @returns {*} the retrieved values
	 */
	var callNestedProperty = function callNestedProperty(path, context, parameters, object) {
	  var parsedPath = getParsedPath(path);

	  return parsedPath.length === 1 ? object ? callIfFunction(object[parsedPath[0]], context, parameters) : void 0 : onMatchAtPath(parsedPath, object, function (ref, key) {
	    return callIfFunction(ref[key], context, parameters);
	  });
	};

	/**
	 * @function getNestedProperty
	 *
	 * @description
	 * parse the path passed and get the nested property at that path
	 *
	 * @param {Array<number|string>|number|string} path the path to retrieve values from the object
	 * @param {*} object the object to get values from
	 * @param {*} noMatchValue an optional fallback value to be returned when the nested property isn't found
	 * @returns {*} the retrieved values
	 */
	var getNestedProperty = function getNestedProperty(path, object, noMatchValue) {
	  var parsedPath = getParsedPath(path);

	  return parsedPath.length === 1 ? object ? getCoalescedValue(object[parsedPath[0]], noMatchValue) : noMatchValue : onMatchAtPath(parsedPath, object, function (ref, key) {
	    return getCoalescedValue(ref[key], noMatchValue);
	  }, false, noMatchValue);
	};

	/**
	 * @function getDeepClone
	 *
	 * @description
	 * parse the path passed and clone the object at that path
	 *
	 * @param {Array<number|string>|number|string} path the path to deeply modify the object on
	 * @param {Array<*>|Object} object the objeisCurrentKeyArrayct to modify
	 * @param {function} onMatch the callback to execute
	 * @returns {Array<*>|Object} the clone object
	 */
	var getDeepClone = function getDeepClone(path, object, onMatch) {
	  var parsedPath = getParsedPath(path);
	  var topLevelClone = isCloneable(object) ? getShallowClone(object) : getNewEmptyChild(parsedPath[0]);

	  if (parsedPath.length === 1) {
	    onMatch(topLevelClone, parsedPath[0]);

	    return topLevelClone;
	  }

	  return onMatchAtPath(parsedPath, topLevelClone, onMatch, true);
	};

	/**
	 * @function hasNestedProperty
	 *
	 * @description
	 * parse the path passed and determine if a value at the path exists
	 *
	 * @param {Array<number|string>|number|string} path the path to retrieve values from the object
	 * @param {*} object the object to get values from
	 * @returns {boolean} does the nested path exist
	 */
	var hasNestedProperty = function hasNestedProperty(path, object) {
	  return getNestedProperty(path, object) !== void 0;
	};

	/* eslint-disable eqeqeq */
	/**
	 * @function isEmptyPath
	 *
	 * @description
	 * is the object passed an empty key value
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object an empty key value
	 */
	var isEmptyPath = function isEmptyPath(object) {
	  return object == null || isArray(object) && !object.length;
	};
	/* eslint-enable */

	/**
	 * @function splice
	 *
	 * @description
	 * splice a single item from the array
	 *
	 * @param {Array<*>} array array to splice from
	 * @param {number} splicedIndex index to splice at
	 */
	var splice = function splice(array, splicedIndex) {
	  if (array.length) {
	    var length = array.length;


	    var index = splicedIndex;

	    while (index < length) {
	      array[index] = array[index + 1];

	      index++;
	    }

	    array.length--;
	  }
	};

	// external dependencies

	/**
	 * @function call
	 *
	 * @description
	 * call a nested method at the path requested with the parameters provided
	 *
	 * @param {Array<number|string>|null|number|string} path the path to get the value at
	 * @param {Array<*>} parameters the parameters to call the method with
	 * @param {Array<*>|Object} object the object to call the method from
	 * @param {*} context the context to set as "this" in the function call
	 */
	var call = curry$1(function (path, parameters, object) {
	  var context = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : object;
	  return isEmptyPath(path) ? callIfFunction(object, context, parameters) : callNestedProperty(path, context, parameters, object);
	},
	// eslint-disable-next-line no-magic-numbers
	3);

	/**
	 * @function get
	 *
	 * @description
	 * get the value to the object at the path requested
	 *
	 * @param {Array<number|string>|null|number|string} path the path to get the value at
	 * @param {Array<*>|Object} object the object to get the value from
	 * @returns {*} the value requested
	 */
	var get = curry$1(function (path, object) {
	  return isEmptyPath(path) ? object : getNestedProperty(path, object);
	});

	/**
	 * @function getOr
	 *
	 * @description
	 * get the value to the object at the path requested, or noMatchValue if nothing
	 * is there.
	 *
	 * @param {*} noMatchValue the fallback value if nothing is found at the given path
	 * @param {Array<number|string>|null|number|string} path the path to get the value at
	 * @param {Array<*>|Object} object the object to get the value from
	 * @returns {*} the value requested
	 */
	var getOr = curry$1(function (noMatchValue, path, object) {
	  return isEmptyPath(path) ? object : getNestedProperty(path, object, noMatchValue);
	});

	/**
	 * @function has
	 *
	 * @description
	 * does the nested path exist on the object
	 *
	 * @param {Array<number|string>|null|number|string} path the path to match on the object
	 * @param {Array<*>|Object} object the object to get the value from
	 * @returns {boolean} does the path exist
	 */
	/* eslint-disable eqeqeq */
	var has = curry$1(function (path, object) {
	  return isEmptyPath(path) ? object != null : hasNestedProperty(path, object);
	});
	/* eslint-enable */

	/**
	 * @function merge
	 *
	 * @description
	 * get the deeply-merged object at path
	 *
	 * @param {Array<number|string>|null|number|string} path the path to match on the object
	 * @param {Array<*>|Object} object the object to merge
	 * @param {Array<*>|Object} object the object to merge with
	 * @returns {Array<*>|Object} the new merged object
	 */
	var merge = curry$1(function (path, objectToMerge, object) {
	  if (!isCloneable(object)) {
	    return objectToMerge;
	  }

	  return isEmptyPath(path) ? getDeeplyMergedObject(object, objectToMerge) : getDeepClone(path, object, function (ref, key) {
	    ref[key] = getDeeplyMergedObject(ref[key], objectToMerge);
	  });
	});

	/**
	 * @function removeobject with quoted keys
	 *
	 * @description
	 * remove the value in the object at the path requested
	 *
	 * @param {Array<number|string>|number|string} path the path to remove the value at
	 * @param {Array<*>|Object} object the object to remove the value from
	 * @returns {Array<*>|Object} a new object with the same structure and the value removed
	 */
	var remove = curry$1(function (path, object) {
	  if (isEmptyPath(path)) {
	    return getNewEmptyObject(object);
	  }

	  return hasNestedProperty(path, object) ? getDeepClone(path, object, function (ref, key) {
	    if (isArray(ref)) {
	      splice(ref, key);
	    } else {
	      delete ref[key];
	    }
	  }) : object;
	});

	/**
	 * @function set
	 *
	 * @description
	 * set the value in the object at the path requested
	 *
	 * @param {Array<number|string>|number|string} path the path to set the value at
	 * @param {*} value the value to set
	 * @param {Array<*>|Object} object the object to set the value in
	 * @returns {Array<*>|Object} a new object with the same structure and the value assigned
	 */
	var set = curry$1(function (path, value, object) {
	  return isEmptyPath(path) ? value : getDeepClone(path, object, function (ref, key) {
	    ref[key] = value;
	  });
	});

	/**
	 * @function add
	 *
	 * @description
	 * add the value to the object at the path requested
	 *
	 * @param {Array<number|string>|null|number|string} path the path to assign the value at
	 * @param {*} value the value to assign
	 * @param {Array<*>|Object} object the object to assignobject the value in
	 * @returns {Array<*>|Object} a new object with the same structure and the value added
	 */
	var add = curry$1(function (path, value, object) {
	  var nestedValue = get(path, object);
	  var fullPath = isArray(nestedValue) ? isArray(path) ? path.concat([nestedValue.length]) : (isEmptyPath(path) ? '' : path) + '[' + nestedValue.length + ']' : path;

	  return set(fullPath, value, object);
	});

	// classes

	/**
	 * @function isArray
	 *
	 * @description
	 * is the object passed an array
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object an array
	 */
	var isArray$1 = Array.isArray;

	/**
	 * @function isCrio
	 *
	 * @description
	 * is the object passed a CrioArray or CrioObject
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object a Crio*
	 */
	var isCrio = function isCrio(object) {
	  return object instanceof CrioArray || object instanceof CrioObject;
	};

	/**
	 * @function isEqual
	 *
	 * @description
	 * are the crio objects equal
	 *
	 * @param {CrioArray|CrioObject} crio crio object to test against
	 * @param {*} object object to test equality with crio object for
	 * @returns {boolean} are the objects equal
	 */
	var isEqual = function isEqual(crio, object) {
	  return isCrio(object) && crio.hashCode === object.hashCode;
	};

	/**
	 * @function isReactElement
	 *
	 * @description
	 * is the object passed a react element
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object a react element
	 */
	var isReactElement = function isReactElement(object) {
	  return !!object && object.$$typeof === REACT_ELEMENT_TYPE;
	};

	/**
	 * @function isObject
	 *
	 * @description
	 * is the object passed a plain object
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object a  plain object
	 */
	var isObject = function isObject(object) {
	  return !!object && object.constructor === Object && !isReactElement(object);
	};

	/**
	 * @function isUndefined
	 *
	 * @description
	 * is the object passed undefined
	 *
	 * @param {*} object the object to test
	 * @returns {boolean} is the object undefined
	 */
	var isUndefined = function isUndefined(object) {
	  return object === void 0;
	};

	var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

	/**
	 * @function createIterator
	 *
	 * @description
	 * create a new iterator method
	 *
	 * @returns {function} the iterator method
	 */
	var createIterator = function createIterator() {
	  return function iterator() {
	    var _this = this;

	    var keys = this.keys();
	    var length = keys.length;

	    var index = 0,
	        value = void 0;

	    return {
	      next: function next() {
	        if (index < length) {
	          value = _this[keys[index++]];

	          return {
	            done: false,
	            value: value
	          };
	        }

	        return {
	          done: true
	        };
	      }
	    };
	  };
	};

	/**
	 * @function every
	 *
	 * @description
	 * does every result from calling fn match
	 *
	 * @param {CrioArray|CrioObject} object the object to test
	 * @param {function} fn the function to perform the test on each item with
	 * @returns {boolean} does every item match
	 */
	var every = function every(object, fn) {
	  var objectKeys = object.keys();

	  var key = void 0;

	  for (var index = 0; index < objectKeys.length; index++) {
	    key = objectKeys[index];

	    if (!fn(object[key], key, object)) {
	      return false;
	    }
	  }

	  return true;
	};

	/**
	 * @function find
	 *
	 * @description
	 * find an item in the array if it exists
	 *
	 * @param {CrioArray|CrioObject} object the object to search
	 * @param {function} fn function to test for finding the item
	 * @param {boolean} [isKey] is the search for a key
	 * @param {boolean} [isFromEnd] is the search for a key
	 * @returns {*} found item or undefined
	 */
	var find = function find(object, fn, isKey, isFromEnd) {
	  var objectKeys = isFromEnd ? object.keys().reverse() : object.keys();

	  var key = void 0;

	  for (var index = 0; index < objectKeys.length; index++) {
	    key = objectKeys[index];

	    if (fn(object[key], key, object)) {
	      return isKey ? key : object[key];
	    }
	  }

	  return isKey && object.isArray() ? -1 : undefined;
	};

	/**
	 * @function getCrioedObject
	 *
	 * @description
	 * get the object converted to a CrioArray or CrioObject, if applicable
	 *
	 * @param {*} object the object to potentially crio
	 * @returns {*} either the crioed object, or the object itself
	 */
	var getCrioedObject = function getCrioedObject(object) {
	  return object && (typeof object === 'undefined' ? 'undefined' : _typeof(object)) === 'object' ? isArray$1(object) ? object instanceof CrioArray ? object : new CrioArray(object) : isObject(object) ? new CrioObject(object) : object : object;
	};

	/**
	 * @function getEntries
	 *
	 * @description
	 * get the [key, value] pairs of the object
	 *
	 * @param {CrioArray|CrioObject} object the object to get the entries of
	 * @returns {CrioArray} the entries of the object
	 */
	var getEntries = function getEntries(object) {
	  return object.keys().map(function (key) {
	    return [key, object[key]];
	  });
	};

	/**
	 * @function getRelativeValue
	 *
	 * @description
	 * get the relative value used in copyWithin
	 *
	 * @param {number} value value used as baseline
	 * @param {number} length the length of the crio
	 * @returns {number} the relative number value
	 */
	var getRelativeValue = function getRelativeValue(value, length) {
	  return value < 0 ? Math.max(length + value, 0) : Math.min(value, length);
	};

	/**
	 * @function getValues
	 *
	 * @description
	 * get the values of the object
	 *
	 * @param {CrioArray|CrioObject} object the object to get the values of
	 * @returns {CrioArray} the values of the object
	 */
	var getValues = function getValues(object) {
	  return object.keys().map(function (key) {
	    return object[key];
	  });
	};

	/**
	 * @function some
	 *
	 * @description
	 * does any result from calling fn match
	 *
	 * @param {CrioArray|CrioObject} object the object to test
	 * @param {function} fn the function to perform the test on each item with
	 * @returns {boolean} does any item match
	 */
	var some = function some(object, fn) {
	  var objectKeys = object.keys();

	  var key = void 0;

	  for (var index = 0; index < objectKeys.length; index++) {
	    key = objectKeys[index];

	    if (fn(object[key], key, object)) {
	      return true;
	    }
	  }

	  return false;
	};

	/**
	 * @function thaw
	 *
	 * @description
	 * convert the CrioArray or CrioObject passed to a plain JS object
	 *
	 * @param {CrioArray|CrioObject} object the object to convert
	 * @returns {Array|Object} the plain JS version of the object passed
	 */
	var thaw = function thaw(object) {
	  return isCrio(object) ? object.isArray() ? [].concat(object).map(function (item) {
	    return thaw(item);
	  }) : Object.keys(object).reduce(function (reducedObject, key) {
	    reducedObject[key] = thaw(object[key]);

	    return reducedObject;
	  }, {}) : object;
	};

	// utils

	/**
	 * @constant {Object} ARRAY_UNSCOPABLES
	 */
	var ARRAY_UNSCOPABLES = {
	  copyWithin: true,
	  entries: true,
	  fill: true,
	  find: true,
	  findIndex: true,
	  findLastIndex: true,
	  includes: true,
	  keys: true,
	  values: true
	};

	/**
	 * @constant {Object} ARRAY_FALLBACK_PROTOTYPE_METHODS
	 */
	var ARRAY_FALLBACK_PROTOTYPE_METHODS = {
	  /**
	   * @function every
	   *
	   * @description
	   * does every instance in the array match
	   *
	   * @param {function} fn the function to test for matching
	   * @returns {boolean} does every instance match
	   */
	  every: function every$$1(fn) {
	    return every(this, fn);
	  },


	  /**
	   * @function find
	   *
	   * @description
	   * find an item in the array if it exists
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {*} found item or undefined
	   */
	  find: function find$$1(fn) {
	    return find(this, fn);
	  },


	  /**
	   * @function findIndex
	   *
	   * @description
	   * find the index of an item in the array if it exists
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {number} index of match, or -1
	   */
	  findIndex: function findIndex(fn) {
	    return find(this, fn, true);
	  },


	  /**
	   * @function includes
	   *
	   * @description
	   * does the array have the item passed
	   *
	   * @param {*} item item to test for existence
	   * @returns {boolean} does the item exist in the array
	   */
	  includes: function includes(item) {
	    return !!~this.indexOf(item);
	  },


	  /**
	   * @function some
	   *
	   * @description
	   * does any item in the array match the result from fn
	   *
	   * @param {function} fn the function to test for matching
	   * @returns {boolean} does any item match
	   */
	  some: function some$$1(fn) {
	    return some(this, fn);
	  }
	};

	/**
	 * @constant {Object} ARRAY_UNSCOPABLES
	 */
	var OBJECT_UNSCOPABLES = {
	  entries: true,
	  fill: true,
	  find: true,
	  findKey: true,
	  findLastKey: true,
	  includes: true,
	  keys: true,
	  values: true
	};

	/**
	 * @constant {Symbol|number}
	 */
	var REACT_ELEMENT_TYPE = typeof Symbol === 'function' && Symbol.for ? Symbol.for('react.element') : 0xeac7;

	var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

	function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

	var hasAppliedPrototype = void 0;

	var CrioObject = function () {
	  function CrioObject(object) {
	    _classCallCheck(this, CrioObject);

	    if (!hasAppliedPrototype) {
	      applyPrototype(); // eslint-disable-line no-use-before-define

	      hasAppliedPrototype = true;
	    }

	    if (isCrio(object)) {
	      return object.toObject();
	    }

	    var objectKeys = isObject(object) ? Object.keys(object) : [];

	    return objectKeys.reduce(function (crioObject, key) {
	      crioObject[key] = getCrioedObject(object[key]);

	      return crioObject;
	    }, this);
	  }

	  /**
	   * @function clear
	   * @memberof CrioObject
	   *
	   * @description
	   * get a new crio that is empty
	   *
	   * @returns {CrioObject} an empty object
	   */
	  CrioObject.prototype.clear = function clear() {
	    return new CrioObject();
	  };

	  /**
	   * @function compact
	   * @memberof CrioObject
	   *
	   * @description
	   * get a new object with values from the original array that are truthy
	   *
	   * @returns {CrioObject} the object with only truthy values
	   */


	  CrioObject.prototype.compact = function compact() {
	    return this.filter(function (item) {
	      return !!item;
	    });
	  };

	  /**
	   * @function delete
	   * @memberof CrioObject
	   *
	   * @description
	   * delete the value in the object at key, either shallow or deep
	   *
	   * @param {Array<number|string>|sring} key the key to delete
	   * @returns {CrioObject} the array with the key deleted
	   */


	  CrioObject.prototype.delete = function _delete(key) {
	    return remove(key, this);
	  };

	  /**
	   * @function entries
	   * @memberof CrioObject
	   *
	   * @description
	   * get the pairs of [key, value] in the crio
	   *
	   * @returns {CrioArray} [key, value] pairs
	   */


	  CrioObject.prototype.entries = function entries() {
	    return getEntries(this);
	  };

	  /**
	   * @function equals
	   * @memberof CrioObject
	   *
	   * @description
	   * does the object passed equal the crio
	   *
	   * @param {*} object object to compare against the instance
	   * @returns {boolean} is the object equivalent in value
	   */


	  CrioObject.prototype.equals = function equals(object) {
	    return isEqual(this, object);
	  };

	  /**
	   * @function every
	   * @memberof CrioObject
	   *
	   * @description
	   * does every instance in the object match
	   *
	   * @param {function} fn the function to test for matching
	   * @returns {boolean} does every instance match
	   */


	  CrioObject.prototype.every = function every$$1(fn) {
	    return every(this, fn);
	  };

	  /**
	   * @function filter
	   * @memberof CrioObject
	   *
	   * @description
	   * filter the object based on the fn passed
	   *
	   * @param {function} fn function to test for if it should be included in the result set
	   * @returns {CrioObject} new crio instance
	   */


	  CrioObject.prototype.filter = function filter(fn) {
	    var _this = this;

	    return new CrioObject(Object.keys(this).reduce(function (object, key) {
	      if (fn(_this[key], key, object)) {
	        object[key] = _this[key];
	      }

	      return object;
	    }, {}));
	  };

	  /**
	   * @function find
	   * @memberof CrioObject
	   *
	   * @description
	   * find an item in the crio if it exists
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {*} found item or undefined
	   */


	  CrioObject.prototype.find = function find$$1(fn) {
	    return find(this, fn);
	  };

	  /**
	   * @function findKey
	   * @memberof CrioObject
	   *
	   * @description
	   * find the key of an item in the crio if it exists
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {number} index of match, or -1
	   */


	  CrioObject.prototype.findKey = function findKey(fn) {
	    return find(this, fn, true);
	  };

	  /**
	   * @function findLast
	   * @memberof CrioObject
	   *
	   * @description
	   * find an item in the crio if it exists, starting from the end and iteratng to the start
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {*} found item or undefined
	   */


	  CrioObject.prototype.findLast = function findLast(fn) {
	    return find(this, fn, false, true);
	  };

	  /**
	   * @function findLastKey
	   * @memberof CrioObject
	   *
	   * @description
	   * find the matching index based on truthy return from fn starting from end
	   *
	   * @param {function} fn function to use for test in iteration
	   * @returns {number} index of match, or -1
	   */


	  CrioObject.prototype.findLastKey = function findLastKey(fn) {
	    return find(this, fn, true, true);
	  };

	  /**
	   * @function forEach
	   * @memberof CrioObject
	   *
	   * @description
	   * iterate over the object calling fn
	   *
	   * @param {function} fn function to call in iteration
	   * @returns {CrioObject} the original object
	   */


	  CrioObject.prototype.forEach = function forEach(fn) {
	    var _this2 = this;

	    Object.keys(this).forEach(function (key) {
	      return fn(_this2[key], key, _this2);
	    });

	    return this;
	  };

	  /**
	   * @function get
	   * @memberof CrioObject
	   *
	   * @description
	   * get the item at key passed, either shallow or deeply nested
	   *
	   * @param {Array<number|string>|string} key key to retrieve
	   * @returns {*} item found at key
	   */


	  CrioObject.prototype.get = function get$$1(key) {
	    return get(key, this);
	  };

	  /**
	   * @function has
	   * @memberof CrioObject
	   *
	   * @description
	   * does the crio have the key passed, either shallow or deeply nested
	   *
	   * @param {Array<number|string>|string} key key to test
	   * @returns {boolean} does the crio have the key
	   */


	  CrioObject.prototype.has = function has$$1(key) {
	    return has(key, this);
	  };

	  /**
	   * @function includes
	   * @memberof CrioObject
	   *
	   * @description
	   * does the object have the item passed
	   *
	   * @param {*} item item to test for existence
	   * @returns {boolean} does the item exist in the crio
	   */


	  CrioObject.prototype.includes = function includes(item) {
	    return this.some(function (value) {
	      return value === item;
	    });
	  };

	  /**
	   * @function isArray
	   * @memberof CrioObject
	   *
	   * @description
	   * is the crio an array
	   *
	   * @returns {boolean} is the crio an array
	   */


	  CrioObject.prototype.isArray = function isArray() {
	    return false;
	  };

	  /**
	   * @function isObject
	   * @memberof CrioObject
	   *
	   * @description
	   * is the crio an object
	   *
	   * @returns {boolean} is the crio an object
	   */


	  CrioObject.prototype.isObject = function isObject$$1() {
	    return true;
	  };

	  /**
	   * @function keyOf
	   * @memberof CrioObject
	   *
	   * @description
	   * get the key for the item passed
	   *
	   * @param {*} item the item to search for
	   * @returns {string} the key of match, or undefined
	   */


	  CrioObject.prototype.keyOf = function keyOf(item) {
	    return this.findKey(function (value) {
	      return value === item;
	    });
	  };

	  /**
	   * @function keys
	   * @memberof CrioObject
	   *
	   * @description
	   * get the keys of the crio
	   *
	   * @returns {CrioArray<string>} keys in the crio
	   */


	  CrioObject.prototype.keys = function keys() {
	    return new CrioArray(Object.keys(this));
	  };

	  /**
	   * @function lastKeyOf
	   * @memberof CrioObject
	   *
	   * @description
	   * get the key for the item passed, starting from the end of the array and iterating towards the start
	   *
	   * @param {*} item the item to search for
	   * @returns {string} the key of match, or undefined
	   */


	  CrioObject.prototype.lastKeyOf = function lastKeyOf(item) {
	    return this.findLastKey(function (value) {
	      return value === item;
	    });
	  };

	  /**
	   * @function map
	   * @memberof CrioObject
	   *
	   * @description
	   * iterate over the object mapping the result of fn to the key
	   *
	   * @param {function} fn function to call on iteration
	   * @returns {Crio} the mapped object
	   */


	  CrioObject.prototype.map = function map(fn) {
	    var _this3 = this;

	    return Object.keys(this).reduce(function (object, key) {
	      object[key] = getCrioedObject(fn(_this3[key], key, _this3));

	      return object;
	    }, new CrioObject({}));
	  };

	  /**
	   * @function merge
	   * @memberof CrioObject
	   *
	   * @description
	   * merge objects with the original object
	   *
	   * @param {Array<number|string>|number|null} key the key to merge into
	   * @param {...Array<CrioObject>} objects objects to merge with the crio
	   * @returns {CrioObject} new crio instance
	   */


	  CrioObject.prototype.merge = function merge$$1(key) {
	    for (var _len = arguments.length, objects = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
	      objects[_key - 1] = arguments[_key];
	    }

	    return objects.reduce(function (mergedObject, object) {
	      return merge(key, getCrioedObject(object), mergedObject);
	    }, this);
	  };

	  /**
	   * @function mutate
	   * @memberof CrioObject
	   *
	   * @description
	   * work with the object in a mutated way and return the crioed result of that call
	   *
	   * @param {function} fn function to apply to crio
	   * @returns {*} crioed value resulting from the call
	   */


	  CrioObject.prototype.mutate = function mutate(fn) {
	    return getCrioedObject(fn(this.thaw(), this));
	  };

	  /**
	   * @function pluck
	   * @memberof CrioObject
	   *
	   * @description
	   * get the values in each object in the collection at key, either shallow or deeply nested
	   *
	   * @param {string} key key to find value of in collection object
	   * @returns {CrioArray} array of plucked values
	   */


	  CrioObject.prototype.pluck = function pluck(key) {
	    var parsedKey = parse(key);
	    var objectToPluck = get(parsedKey.slice(0, parsedKey.length - 1), this);
	    var finalKey = parsedKey.slice(-1);

	    return objectToPluck.map(function (item) {
	      return get(finalKey, item);
	    }).values();
	  };

	  /**
	   * @function reduce
	   * @memberof CrioObject
	   *
	   * @description
	   * reduce the crio down to a single value, starting with initial value
	   *
	   * @param {function} fn the function to iterate with
	   * @param {*} initialValue the initial value of the reduction
	   * @returns {*} the reduced value
	   */


	  CrioObject.prototype.reduce = function reduce(fn, initialValue) {
	    var _this4 = this;

	    return getCrioedObject(Object.keys(this).reduce(function (value, key) {
	      return fn(value, _this4[key], key, _this4);
	    }, initialValue));
	  };

	  /**
	   * @function reduceRight
	   * @memberof CrioObject
	   *
	   * @description
	   * reduce the crio down to a single value, starting with initial value, starting from the end of the array
	   * and iterating to the start
	   *
	   * @param {function} fn the function to iterate with
	   * @param {*} initialValue the initial value of the reduction
	   * @returns {*} the reduced value
	   */


	  CrioObject.prototype.reduceRight = function reduceRight(fn, initialValue) {
	    var _this5 = this;

	    return getCrioedObject(Object.keys(this).reduceRight(function (value, key) {
	      return fn(value, _this5[key], key, _this5);
	    }, initialValue));
	  };

	  /**
	   * @function set
	   * @memberof CrioObject
	   *
	   * @description
	   * set the value at the key passed
	   *
	   * @param {Array<number|string>|string} key key to assign value to
	   * @param {*} value value to assign
	   * @returns {CrioObject} object with value set at key
	   */


	  CrioObject.prototype.set = function set$$1(key, value) {
	    return set(key, getCrioedObject(value), this);
	  };

	  /**
	   * @function some
	   * @memberof CrioObject
	   *
	   * @description
	   * does any item in the object match the result from fn
	   *
	   * @param {function} fn the function to test for matching
	   * @returns {boolean} does any item match
	   */


	  CrioObject.prototype.some = function some$$1(fn) {
	    return some(this, fn);
	  };

	  /**
	   * @function sort
	   * @memberof CrioObject
	   *
	   * @description
	   * sort the collection by the fn passed
	   *
	   * @param {function} fn the function to sort based on
	   * @returns {CrioObject} object with the items sorted
	   */


	  CrioObject.prototype.sort = function sort(fn) {
	    var _this6 = this;

	    return new CrioObject(Object.keys(this).sort(fn).reduce(function (object, key) {
	      object[key] = _this6[key];

	      return object;
	    }, {}));
	  };

	  /**
	   * @function thaw
	   * @memberof CrioObject
	   *
	   * @description
	   * create a plain JS version of the object
	   *
	   * @returns {Object} plain JS version of the object
	   */


	  CrioObject.prototype.thaw = function thaw$$1() {
	    return thaw(this);
	  };

	  /**
	   * @function toArray
	   * @memberof CrioObject
	   *
	   * @description
	   * convert the object to an array
	   *
	   * @returns {CrioArray} the object converted to an array of its values
	   */


	  CrioObject.prototype.toArray = function toArray() {
	    return this.values();
	  };

	  /**
	   * @function toLocaleString
	   * @memberof CrioObject
	   *
	   * @description
	   * convert the object to stringified form
	   *
	   * @param {function} [serializer] the serialization method to use
	   * @param {number} [indent] the number of spaces to indent the values
	   * @returns {string} stringified object
	   */


	  CrioObject.prototype.toLocaleString = function toLocaleString(serializer, indent) {
	    return this.toString(serializer, indent);
	  };

	  /**
	   * @function toObject
	   * @memberof CrioObject
	   *
	   * @description
	   * convert the object to an objectobject
	   *
	   * @returns {CrioObject} the object
	   */


	  CrioObject.prototype.toObject = function toObject() {
	    return this;
	  };

	  /**
	   * @function toString
	   * @memberof CrioObject
	   *
	   * @description
	   * convert the object to stringified form
	   *
	   * @param {function} [serializer] the serialization method to use
	   * @param {number} [indent] the number of spaces to indent the values
	   * @returns {string} stringified object
	   */


	  CrioObject.prototype.toString = function toString(serializer, indent) {
	    return stringify_1(this, serializer, indent);
	  };

	  /**
	   * @function valueOf
	   * @memberof CrioObject
	   *
	   * @description
	   * get the object value
	   *
	   * @returns {CrioObject} the object
	   */


	  CrioObject.prototype.valueOf = function valueOf() {
	    return this;
	  };

	  /**
	   * @function values
	   * @memberof CrioObject
	   *
	   * @description
	   * get the values of the object as an array
	   *
	   * @returns {CrioObject} values in the object
	   */


	  CrioObject.prototype.values = function values() {
	    return getValues(this);
	  };

	  _createClass(CrioObject, [{
	    key: 'hashCode',
	    get: function get$$1() {
	      return hash(this, true);
	    }
	  }, {
	    key: 'size',
	    get: function get$$1() {
	      return Object.keys(this).length;
	    }
	  }]);

	  return CrioObject;
	}();

	function applyPrototype() {
	  if (typeof Symbol === 'function') {
	    if (Symbol.species) {
	      Object.defineProperty(CrioObject, Symbol.species, {
	        configurable: false,
	        enumerable: false,
	        get: function get$$1() {
	          return CrioObject;
	        }
	      });
	    }

	    if (Symbol.iterator) {
	      Object.defineProperty(CrioObject.prototype, Symbol.iterator, {
	        configurable: false,
	        enumerable: false,
	        value: createIterator(),
	        writable: false
	      });
	    }

	    if (Symbol.unscopables) {
	      Object.defineProperty(CrioObject.prototype, Symbol.unscopables, {
	        configurable: false,
	        enumerable: false,
	        value: OBJECT_UNSCOPABLES,
	        writable: false
	      });
	    }
	  }
	}

	var _createClass$1 = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

	function _classCallCheck$1(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

	function _possibleConstructorReturn(self, call$$1) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call$$1 && (typeof call$$1 === "object" || typeof call$$1 === "function") ? call$$1 : self; }

	function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

	function _extendableBuiltin(cls) {
	  function ExtendableBuiltin() {
	    var instance = Reflect.construct(cls, Array.from(arguments));
	    Object.setPrototypeOf(instance, Object.getPrototypeOf(this));
	    return instance;
	  }

	  ExtendableBuiltin.prototype = Object.create(cls.prototype, {
	    constructor: {
	      value: cls,
	      enumerable: false,
	      writable: true,
	      configurable: true
	    }
	  });

	  if (Object.setPrototypeOf) {
	    Object.setPrototypeOf(ExtendableBuiltin, cls);
	  } else {
	    ExtendableBuiltin.__proto__ = cls;
	  }

	  return ExtendableBuiltin;
	}

	var hasAppliedPrototype$1 = false;

	var CrioArray = function (_extendableBuiltin2) {
	  _inherits(CrioArray, _extendableBuiltin2);

	  function CrioArray(array) {
	    var _ret;

	    _classCallCheck$1(this, CrioArray);

	    var _this = _possibleConstructorReturn(this, _extendableBuiltin2.call(this, isArray$1(array) ? array.length : array || 0));

	    if (!hasAppliedPrototype$1) {
	      applyPrototype$1(); // eslint-disable-line no-use-before-define

	      hasAppliedPrototype$1 = true;
	    }

	    return _ret = isCrio(array) ? array.toArray() : isArray$1(array) ? array.reduce(function (crioArray, item, index) {
	      crioArray[index] = getCrioedObject(item);

	      return crioArray;
	    }, _this) : _this, _possibleConstructorReturn(_this, _ret);
	  }

	  /**
	   * @function clear
	   * @memberof CrioArray
	   *folder
	   * @description
	   * get a new empty array
	   *
	   * @returns {CrioArray} the empty array
	   */
	  CrioArray.prototype.clear = function clear() {
	    return new CrioArray();
	  };

	  /**
	   * @function compact
	   * @memberof CrioArray
	   *
	   * @description
	   * get a new array with values from the original array that are truthy
	   *
	   * @returns {CrioArray} the array with only truthy values
	   */


	  CrioArray.prototype.compact = function compact() {
	    return this.filter(function (item) {
	      return !!item;
	    });
	  };

	  /**
	   * @function copyWithin
	   *
	   * @description
	   * move values around within the array
	   *
	   * @param {number} targetIndex target to copy
	   * @param {number} [startIndex=0] index to start copying to
	   * @param {number} [endIndex=this.length] index to stop copying to
	   * @returns {CrioArray} array with target copied in appropriate spots
	   */


	  CrioArray.prototype.copyWithin = function copyWithin(targetIndex) {
	    var startIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
	    var endIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.length;

	    var clone = [].concat(this);
	    var length = this.length >>> 0;
	    var final = getRelativeValue(endIndex >> 0, length);

	    var to = getRelativeValue(targetIndex >> 0, length),
	        from = getRelativeValue(startIndex >> 0, length),
	        count = Math.min(final - from, length - to),
	        direction = 1;

	    if (from < to && to < from + count) {
	      direction = -1;
	      from += count - 1;
	      to += count - 1;
	    }

	    while (count > 0) {
	      if (from in clone) {
	        clone[to] = clone[from];
	      } else {
	        delete clone[to];
	      }

	      from += direction;
	      to += direction;
	      count--;
	    }

	    return new this.constructor(clone);
	  };

	  /**
	   * @function delete
	   * @memberof CrioArray
	   *
	   * @description
	   * delete the value in the array at key, either shallow or deep
	   *
	   * @param {Array<number|string>|number} key the key to delete
	   * @returns {CrioArray} the array with the key deleted
	   */


	  CrioArray.prototype.delete = function _delete(key) {
	    return remove(key, this);
	  };

	  /**
	   * @function difference
	   * @memberof CrioArray
	   *
	   * @description
	   * find the values in this that do not exist in any of the arrays passed
	   *
	   * @param {...Array<Array<*>>} arrays arrays to get the difference of
	   * @returns {CrioArray} array of items matching diffference criteria
	   */


	  CrioArray.prototype.difference = function difference() {
	    for (var _len = arguments.length, arrays = Array(_len), _key = 0; _key < _len; _key++) {
	      arrays[_key] = arguments[_key];
	    }

	    if (!arrays.length) {
	      return this;
	    }

	    var indexOfItem = void 0;

	    return arrays.reduce(function (differenceArray, array) {
	      if (isArray$1(array)) {
	        array.forEach(function (item) {
	          indexOfItem = differenceArray.indexOf(item);

	          if (~indexOfItem) {
	            differenceArray = differenceArray.splice(indexOfItem, 1);
	          }
	        });
	      }

	      return differenceArray;
	    }, this);
	  };

	  /**
	   * @function entries
	   * @memberof CrioArray
	   *
	   * @description
	   * get the pairs of [key, value] in the crio
	   *
	   * @returns {CrioArray} [key, value] pairs
	   */


	  CrioArray.prototype.entries = function entries() {
	    return getEntries(this);
	  };

	  /**
	   * @function equals
	   * @memberof CrioArray
	   *
	   * @description
	   * does the object passed equal the crio
	   *
	   * @param {*} object object to compare against the instance
	   * @returns {boolean} is the object equivalent in value
	   */


	  CrioArray.prototype.equals = function equals(object) {
	    return isEqual(this, object);
	  };

	  /**
	   * @function fill
	   *
	   * @description
	   * fill the array at certain indices with the value passed
	   *
	   * @param {*} value the value to fill the indices with
	   * @param {number} [startIndex=0] the starting index to fill
	   * @param {number} [endIndex=this.length] the ending index to fill
	   * @returns {CrioArray} array with values filled appropriately
	   */


	  CrioArray.prototype.fill = function fill(value) {
	    var startIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
	    var endIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.length;

	    var from = startIndex < 0 ? this.length + startIndex : startIndex;
	    var to = endIndex < 0 ? this.length + endIndex : endIndex;
	    var crioedValue = getCrioedObject(value);

	    return this.map(function (item, index) {
	      return index >= from && index < to ? crioedValue : item;
	    });
	  };

	  /**
	   * @function findLast
	   * @memberof CrioArray
	   *
	   * @description
	   * find an item in the crio if it exists, starting from the end and iteratng to the start
	   *
	   * @param {function} fn function to test for finding the item
	   * @returns {*} found item or undefined
	   */


	  CrioArray.prototype.findLast = function findLast(fn) {
	    return find(this, fn, false, true);
	  };

	  /**
	   * @function findLastIndex
	   * @memberof CrioArray
	   *
	   * @description
	   * find the matching index based on truthy return from fn starting from end
	   *
	   * @param {function} fn function to use for test in iteration
	   * @returns {number} index of match, or -1
	   */


	  CrioArray.prototype.findLastIndex = function findLastIndex(fn) {
	    return find(this, fn, true, true);
	  };

	  /**
	   * @function first
	   * @memberof CrioArray
	   *
	   * @description
	   * take the first n number of items in the array
	   *
	   * @param {number} [size=1] size of elements to take from beginning of array
	   * @returns {CrioArray}
	   */


	  CrioArray.prototype.first = function first() {
	    var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

	    return this.slice(0, size);
	  };

	  /**
	   * @function forEach
	   * @memberof CrioArray
	   *
	   * @description
	   * iterate over the array executing fn
	   *
	   * @param {function} fn the function to execute
	   * @returns {CrioArray} the original array
	   */


	  CrioArray.prototype.forEach = function forEach(fn) {
	    Array.prototype.forEach.call(this, fn);

	    return this;
	  };

	  /**
	   * @function get
	   * @memberof CrioArray
	   *
	   * @description
	   * get the item at key passed, either shallow or deeply nested
	   *
	   * @param {Array<number|string>|number} key key to retrieve
	   * @returns {*} item found at key
	   */


	  CrioArray.prototype.get = function get$$1(key) {
	    return get(key, this);
	  };

	  /**
	   * @function has
	   * @memberof CrioArray
	   *
	   * @description
	   * does the crio have the key passed, either shallow or deeply nested
	   *
	   * @param {Array<number|string>|number} key key to test
	   * @returns {boolean} does the crio have the key
	   */


	  CrioArray.prototype.has = function has$$1(key) {
	    return has(key, this);
	  };

	  /**
	   * @function intersection
	   * @memberof CrioArray
	   *
	   * @description
	   * find the values in that exist in this and each of the arrays passed
	   *
	   * @param {...Array<Array>} arrays to find the intersecting values of
	   * @returns {CrioArray} array of values that exist in all arrays
	   */


	  CrioArray.prototype.intersection = function intersection() {
	    for (var _len2 = arguments.length, arrays = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
	      arrays[_key2] = arguments[_key2];
	    }

	    if (!arrays.length) {
	      return this;
	    }

	    var indices = [],
	        indexOfItem = void 0;

	    var reducedArrays = [this].concat(arrays).reduce(function (items, array) {
	      if (isArray$1(array)) {
	        array.forEach(function (item) {
	          indexOfItem = items.indexOf(item);

	          if (~indexOfItem) {
	            return indices[indexOfItem]++;
	          }

	          indices[items.length] = 1;
	          items.push(item);
	        });
	      }

	      return items;
	    }, []);

	    var newLength = arrays.length + 1;

	    return new CrioArray(reducedArrays.filter(function (itemIgnored, index) {
	      return indices[index] === newLength;
	    }));
	  };

	  /**
	   * @function isArray
	   * @memberof CrioArray
	   *
	   * @description
	   * is the crio an array
	   *
	   * @returns {boolean} is the crio an array
	   */


	  CrioArray.prototype.isArray = function isArray() {
	    return true;
	  };

	  /**
	   * @function isObject
	   * @memberof CrioArray
	   *
	   * @description
	   * is the crio an object
	   *
	   * @returns {boolean} is the crio an object
	   */


	  CrioArray.prototype.isObject = function isObject$$1() {
	    return false;
	  };

	  /**
	   * @function join
	   * @memberof CrioArray
	   *
	   * @description
	   * join the values in the crio as a string, combined with separator
	   *
	   * @param {string} [separator] character(s) to place between strings in combination
	   * @returns {string} parameters joined by separator in string
	   */


	  CrioArray.prototype.join = function join(separator) {
	    return this.thaw().join(separator);
	  };

	  /**
	   * @function keys
	   * @memberof CrioArray
	   *
	   * @description
	   * get the keys of the crio
	   *
	   * @returns {CrioArray<number>} keys in the crio
	   */


	  CrioArray.prototype.keys = function keys() {
	    return new CrioArray(Object.keys(this).map(Number));
	  };

	  /**
	   * @function last
	   * @memberof CrioArray
	   *
	   * @description
	   * take the last n number of items in the array
	   *
	   * @param {number} [size=1] size of elements to take from end of array
	   * @returns {CrioArray}
	   */


	  CrioArray.prototype.last = function last() {
	    var size = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

	    return this.slice(this.length - size);
	  };

	  /**
	   * @function map
	   * @memberof CrioArray
	   *
	   * @description
	   * map over the array returning the mapped items
	   *
	   * @param {function} fn the function to map
	   * @returns {CrioArray} the mapped array
	   */


	  CrioArray.prototype.map = function map(fn) {
	    var _this2 = this;

	    return Array.prototype.map.call(this, function (item, index) {
	      return getCrioedObject(fn(item, index, _this2));
	    });
	  };

	  /**
	   * @function merge
	   * @memberof CrioArray
	   *
	   * @description
	   * merge arrays with the original array
	   *
	   * @param {Array<number|string>|number|null} key the key to merge into
	   * @param {...Array<CrioArray>} objects objects to merge with the crio
	   * @returns {CrioArray} merged array
	   */


	  CrioArray.prototype.merge = function merge$$1(key) {
	    for (var _len3 = arguments.length, objects = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
	      objects[_key3 - 1] = arguments[_key3];
	    }

	    return objects.reduce(function (mergedObject, object) {
	      return merge(key, getCrioedObject(object), mergedObject);
	    }, this);
	  };

	  /**
	   * @function mutate
	   * @memberof CrioArray
	   *
	   * @description
	   * work with the array in a mutated way and return the crioed result of that call
	   *
	   * @param {function} fn function to apply to crio
	   * @returns {*} crioed value resulting from the call
	   */


	  CrioArray.prototype.mutate = function mutate(fn) {
	    return getCrioedObject(fn(this.thaw(), this));
	  };

	  /**
	   * @function pluck
	   * @memberof CrioArray
	   *
	   * @description
	   * get the values in each object in the collection at key, either shallow or deeply nested
	   *
	   * @param {string} key key to find value of in collection object
	   * @returns {CrioArray} array of plucked values
	   */


	  CrioArray.prototype.pluck = function pluck(key) {
	    var parsedKey = parse(key);
	    var arrayToPluck = get(parsedKey.slice(0, parsedKey.length - 1), this);
	    var finalKey = parsedKey.slice(-1);

	    return arrayToPluck.map(function (item) {
	      return get(finalKey, item);
	    });
	  };

	  /**
	   * @function pop
	   * @memberof CrioArray
	   *
	   * @description
	   * get crio based on current crio with last item removed
	   *
	   * @returns {CrioArray} array with the last value removed
	   */


	  CrioArray.prototype.pop = function pop() {
	    return this.slice(0, this.length - 1);
	  };

	  /**
	   * @function push
	   * @memberof CrioArray
	   *
	   * @description
	   * push one to many items to the current crio
	   *
	   * @param {...Array<*>} items the items to add to the array
	   * @returns {CrioArray} array with the values added
	   */


	  CrioArray.prototype.push = function push() {
	    for (var _len4 = arguments.length, items = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
	      items[_key4] = arguments[_key4];
	    }

	    return items.length ? this.concat(items) : this;
	  };

	  /**
	   * @function reduce
	   * @memberof CrioObject
	   *
	   * @description
	   * reduce the crio down to a single value, starting with initial value
	   *
	   * @param {function} fn the function to iterate with
	   * @param {*} initialValue the initial value of the reduction
	   * @returns {*} the reduced value
	   */


	  CrioArray.prototype.reduce = function reduce(fn, initialValue) {
	    var _this3 = this;

	    return getCrioedObject(Array.prototype.reduce.call(this, function (value, item, index) {
	      return fn(value, item, index, _this3);
	    }, initialValue));
	  };

	  /**
	   * @function reduceRight
	   * @memberof CrioObject
	   *
	   * @description
	   * reduce the crio down to a single value, starting with initial value, starting from the end of the array
	   * and iterating to the start
	   *
	   * @param {function} fn the function to iterate with
	   * @param {*} initialValue the initial value of the reduction
	   * @returns {*} the reduced value
	   */


	  CrioArray.prototype.reduceRight = function reduceRight(fn, initialValue) {
	    var _this4 = this;

	    return getCrioedObject(Array.prototype.reduceRight.call(this, function (value, item, index) {
	      return fn(value, item, index, _this4);
	    }, initialValue));
	  };

	  /**
	   * @function reverse
	   * @memberof CrioArray
	   *
	   * @description
	   * get the same values, but in reverse order
	   *
	   * @returns {CrioArray} array with the items reversed in order
	   */


	  CrioArray.prototype.reverse = function reverse() {
	    return new CrioArray([].concat(this).reverse());
	  };

	  /**
	   * @function set
	   * @memberof CrioArray
	   *
	   * @description
	   * set the value at the key passed
	   *
	   * @param {Array<number|string>|number} key key to assign value to
	   * @param {*} value value to assign
	   * @returns {CrioArray} array with value set at key
	   */


	  CrioArray.prototype.set = function set$$1(key, value) {
	    return set(key, getCrioedObject(value), this);
	  };

	  /**
	   * @function shift
	   * @memberof CrioArray
	   *
	   * @description
	   * get crio based on current crio with first item removed
	   *
	   * @returns {CrioArray} array with the first item removed
	   */


	  CrioArray.prototype.shift = function shift() {
	    return this.slice(1);
	  };

	  /**
	   * @function sort
	   * @memberof CrioArray
	   *
	   * @description
	   * sort the collection by the fn passed
	   *
	   * @param {function} fn the function to sort based on
	   * @returns {CrioArray} array with the items sorted
	   */


	  CrioArray.prototype.sort = function sort(fn) {
	    var clone = [].concat(this);

	    clone.sort(fn);

	    return new CrioArray(clone);
	  };

	  /**
	   * @function splice
	   * @memberof CrioArray
	   *
	   * @description
	   * splice the values into or out of the array
	   *
	   * @param {number} [start=0] starting index to splice
	   * @param {number} [deleteCount=1] length from starting index to removes
	   * @param {...Array<*>} items items to insert after delete is complete
	   * @returns {CrioArray} array with the value spliced in / out
	   */


	  CrioArray.prototype.splice = function splice() {
	    var clone = [].concat(this);

	    clone.splice.apply(clone, arguments);

	    return new CrioArray(clone);
	  };

	  /**
	   * @function thaw
	   * @memberof CrioArray
	   *
	   * @description
	   * create a plain JS version of the array
	   *
	   * @returns {Array<*>} plain JS version of the array
	   */


	  CrioArray.prototype.thaw = function thaw$$1() {
	    return thaw(this);
	  };

	  /**
	   * @function toArray
	   * @memberof CrioArray
	   *
	   * @description
	   * convert the array to an array
	   *
	   * @returns {CrioArray} the array
	   */


	  CrioArray.prototype.toArray = function toArray() {
	    return this;
	  };

	  /**
	   * @function toLocaleString
	   * @memberof CrioArray
	   *
	   * @description
	   * convert the array to stringified form
	   *
	   * @param {function} [serializer] the serialization method to use
	   * @param {number} [indent] the number of spaces to indent the values
	   * @returns {string} stringified array
	   */


	  CrioArray.prototype.toLocaleString = function toLocaleString(serializer, indent) {
	    return this.toString(serializer, indent);
	  };

	  /**
	   * @function toObject
	   * @memberof CrioArray
	   *
	   * @description
	   * convert the crio to an object if it isn't already
	   *
	   * @returns {CrioObject} new object from the array
	   */


	  CrioArray.prototype.toObject = function toObject() {
	    return this.reduce(function (object, item, index) {
	      object[index] = item;

	      return object;
	    }, new CrioObject({}));
	  };

	  /**
	   * @function toString
	   * @memberof CrioArray
	   *
	   * @description
	   * convert the array to stringified form
	   *
	   * @param {function} [serializer] the serialization method to use
	   * @param {number} [indent] the number of spaces to indent the values
	   * @returns {string} stringified array
	   */


	  CrioArray.prototype.toString = function toString(serializer, indent) {
	    return stringify_1(this, serializer, indent);
	  };

	  /**
	   * @function unique
	   * @memberof CrioArray
	   *
	   * @description
	   * return the current CrioArray with the duplicate values removed
	   *
	   * @returns {CrioArray} new crio instance
	   */


	  CrioArray.prototype.unique = function unique() {
	    var hashArray = [],
	        newArray = [],
	        hasHashCode = false,
	        hashCode = void 0,
	        storeValue = void 0;

	    return this.filter(function (item) {
	      hashCode = item ? item.hashCode : undefined;
	      hasHashCode = !isUndefined(hashCode);
	      storeValue = !~newArray.indexOf(item) && (!hasHashCode || !~hashArray.indexOf(hashCode));

	      if (storeValue) {
	        newArray.push(item);

	        if (hasHashCode) {
	          hashArray.push(hashCode);
	        }
	      }

	      return storeValue;
	    });
	  };

	  /**
	   * @function unshift
	   * @memberof CrioArray
	   *
	   * @description
	   * add items passed to the beginning of the crio array
	   *
	   * @param {...Array<*>} items items to prepend to the array
	   * @returns {CrioArray} array with the items prepended
	   */


	  CrioArray.prototype.unshift = function unshift() {
	    for (var _len5 = arguments.length, items = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
	      items[_key5] = arguments[_key5];
	    }

	    return items.length ? new CrioArray([].concat(items, this)) : this;
	  };

	  /**
	   * @function valueOf
	   * @memberof CrioArray
	   *
	   * @description
	   * get the array value
	   *
	   * @returns {CrioArray} the array
	   */


	  CrioArray.prototype.valueOf = function valueOf() {
	    return this;
	  };

	  /**
	   * @function values
	   * @memberof CrioArray
	   *
	   * @description
	   * get the values of the array as an array
	   *
	   * @returns {CrioArray} values in the array
	   */


	  CrioArray.prototype.values = function values() {
	    return getValues(this);
	  };

	  /**
	   * @function xor
	   * @memberof CrioArray
	   *
	   * @description
	   * find the values that are the symmetric difference of this and the arrays passed
	   *
	   * @param {Array<Array>} arrays arrays to find symmetric values in
	   * @returns {CrioArray} array of the symmetric differences of all the arrays
	   */


	  CrioArray.prototype.xor = function xor() {
	    for (var _len6 = arguments.length, arrays = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
	      arrays[_key6] = arguments[_key6];
	    }

	    if (!arrays.length) {
	      return this;
	    }

	    var indicesToRemove = [],
	        indexOfItem = void 0;

	    var reducedItems = [this].concat(arrays).reduce(function (items, array) {
	      if (isArray$1(array)) {
	        array.forEach(function (item) {
	          indexOfItem = items.indexOf(item);

	          if (~indexOfItem) {
	            indicesToRemove.push(indexOfItem);
	          } else {
	            items = items.push(item);
	          }
	        });
	      }

	      return items;
	    }, new CrioArray([]));

	    return reducedItems.filter(function (itemIgnored, index) {
	      return !~indicesToRemove.indexOf(index);
	    });
	  };

	  _createClass$1(CrioArray, [{
	    key: 'hashCode',
	    get: function get$$1() {
	      return hash(this, true);
	    }
	  }]);

	  return CrioArray;
	}(_extendableBuiltin(Array));

	function applyPrototype$1() {
	  Object.keys(ARRAY_FALLBACK_PROTOTYPE_METHODS).forEach(function (key) {
	    if (typeof Array.prototype[key] !== 'function') {
	      CrioArray.prototype[key] = function () {
	        for (var _len7 = arguments.length, args = Array(_len7), _key7 = 0; _key7 < _len7; _key7++) {
	          args[_key7] = arguments[_key7];
	        }

	        return ARRAY_FALLBACK_PROTOTYPE_METHODS[key].apply(ARRAY_FALLBACK_PROTOTYPE_METHODS, [this].concat(args));
	      };
	    }
	  });

	  if (typeof Symbol === 'function') {
	    if (Symbol.species) {
	      Object.defineProperty(CrioArray, Symbol.species, {
	        configurable: false,
	        enumerable: false,
	        get: function get$$1() {
	          return CrioArray;
	        }
	      });
	    }

	    if (Symbol.iterator) {
	      Object.defineProperty(CrioArray.prototype, Symbol.iterator, {
	        configurable: false,
	        enumerable: false,
	        value: createIterator(),
	        writable: false
	      });
	    }

	    if (Symbol.unscopables) {
	      Object.defineProperty(CrioArray.prototype, Symbol.unscopables, {
	        configurable: false,
	        enumerable: false,
	        value: ARRAY_UNSCOPABLES,
	        writable: false
	      });
	    }
	  }
	}

	// classes

	/**
	 * @module crio
	 */

	/**
	 * @function crio
	 *
	 * @description
	 * build a new crio object based on the object passed
	 *
	 * @param {*} [object={}] object to convert to crio
	 *
	 * @returns {CrioArray|CrioObject} object that has been crioed
	 */
	var crio = function crio() {
	  var object = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

	  if (isCrio(object)) {
	    return object;
	  }

	  if (isArray$1(object)) {
	    return new CrioArray(object);
	  }

	  if (isObject(object)) {
	    return new CrioObject(object);
	  }

	  throw new TypeError('Object passed must be either an array or a plain object.');
	};

	/**
	 * @function crio.array
	 *
	 * @description
	 * convenience method for making crio arrays
	 *
	 * @param {Array<*>} array array to crio
	 * @returns {CrioArray} crio array
	 */
	crio.array = function () {
	  var array = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

	  if (!isArray$1(array)) {
	    throw new TypeError('Object passed must be an array.');
	  }

	  return crio(array);
	};

	/**
	 * @function crio.isArray
	 *
	 * @description
	 * check if object is a crio array
	 *
	 * @param {*} object object to test
	 * @returns {boolean} is the object a crio array
	 */
	crio.isArray = function (object) {
	  return isCrio(object) && object.isArray();
	};

	crio.isCrio = isCrio;

	/**
	 * @function crio.isObject
	 *
	 * @description
	 * check if object is a crio object
	 *
	 * @param {*} object object to test
	 * @returns {boolean} is the object a crio object
	 */
	crio.isObject = function (object) {
	  return isCrio(object) && object.isObject();
	};

	/**
	 * @function crio.object
	 *
	 * @description
	 * convenience method for making crio objects
	 *
	 * @param {Object} object object to crio
	 * @returns {CrioObject} crio object
	 */
	crio.object = function () {
	  var object = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

	  if (!isObject(object)) {
	    throw new TypeError('Object passed must be a plain object.');
	  }

	  return crio(object);
	};

	exports.default = crio;

	Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=crio.js.map
